#include "status_ui.hpp"
#include "components.hpp"
#include "inventory.hpp"
#include "color.hpp"
#include "guecs.hpp"
#include "rand.hpp"

namespace gui {
  using namespace guecs;

  StatusUI::StatusUI(GameLevel level) :
    $level(level)
  {
    $gui.position(STATUS_UI_X, STATUS_UI_Y, STATUS_UI_WIDTH, STATUS_UI_HEIGHT);
    $gui.layout(
        "[slot1 | slot2 | slot3]"
        "[slot4 | slot5 | slot6]"
        "[slot7 | slot8 | slot9]"
        "[*%(100,300)log_view]"
        "[_]"
        "[_]");
  }

  void StatusUI::render() {
    for(auto& [name, cell] : $gui.cells()) {
      if(name == "log_view") {
        $log_to = $gui.entity("log_view");
        $gui.set<Rectangle>($log_to, {});
        $gui.set<Textual>($log_to, {"Welcome to the Game!", 20});
      } else {
        auto button = $gui.entity(name);
        $gui.set<Rectangle>(button, {});
        $gui.set<Textual>(button, {""});
        $gui.set<ActionData>(button, {std::make_any<std::string>(name)});
        $gui.set<Clickable>(button, {
            [&](auto ent, auto data){ select_slot(ent, data); }
        });
      }
    }

    $gui.init();
  }

  void StatusUI::select_slot(DinkyECS::Entity ent, std::any) {
    auto cn = $gui.get<CellName>(ent);
    auto world = $level.world;

    if(world->has<components::Inventory>($level.player)) {
      auto& inventory = world->get<components::Inventory>($level.player);
      for(size_t i = 0; i < inventory.count(); i++) {
        if($slots[i] == cn.name) {
          auto& player_combat = world->get<components::Combat>($level.player);
          auto& item = inventory.get(i);

          if(item.count == 0) continue;

          if(item.data["id"] == "SWORD_RUSTY") {
            auto weapon = components::get<components::Weapon>(item.data);
            player_combat.damage = weapon.damage;
            inventory.decrease(i, 1);
            log("You equip a new sword.");
            break;
          } else if(item.data["id"] == "POTION_HEALING_SMALL") {
            auto cure = components::get<components::Curative>(item.data);
            int prev_hp = player_combat.hp;
            player_combat.hp = std::min(player_combat.hp + cure.hp, player_combat.max_hp);
            log(fmt::format("Healed player from {} to {}", prev_hp, player_combat.hp));
            inventory.decrease(i, 1);
            break;
          } else if(item.data["id"] == "TORCH_BAD") {
            auto new_light = components::get<components::LightSource>(item.data);
            world->set<components::LightSource>($level.player, new_light);
            inventory.light = new_light;
            inventory.decrease(i, 1);

            log("You are using a new torch.");
            break;
          }
        }
      }

      update();
    }
  }

  /* WARNING: This is really not the greatest way to do this. */
  void StatusUI::update() {
    if($gui.has<Textual>($log_to)) {
      auto& text = $gui.get<Textual>($log_to);
      std::string log;
      for(auto msg : $messages) {
        log += msg + "\n";
      }
      text.update(log);
    }

    auto world = $level.world;
    if(world->has<components::Inventory>($level.player)) {
      auto& inventory = world->get<components::Inventory>($level.player);

      if(inventory.count() > 0) {
        size_t limit = std::min(inventory.count(), $slots.size());

        for(size_t i = 0; i < limit; i++) {
          auto slot = $gui.entity($slots[i]);
          auto& item = inventory.get(i);
          auto comp_sprite = components::get<components::Sprite>(item.data);
          $gui.set_init<guecs::Sprite>(slot, {comp_sprite.name});
          std::string count_label = fmt::format("{}", item.count);
          auto& label = $gui.get<Textual>(slot);
          label.text->setString(count_label);

          auto& sprite = $gui.get<guecs::Sprite>(slot);

          if(item.count == 0) {
            sprite.sprite->setColor({125, 125, 125});
          } else {
            sprite.sprite->setColor({255, 255, 255});
          }
        }
      }
    }
  }

  void StatusUI::draw(sf::RenderWindow &window) {
    $gui.render(window);
  }

  void StatusUI::log(std::string msg) {
    $messages.push_front(msg);
    if($messages.size() > MAX_LOG_MESSAGES) {
      $messages.pop_back();
    }
  }
}
